/*	Public domain	*/

#define MySetError SF_SetError

#include <config/have_getpeereid.h>
#include <config/have_so_peercred.h>
#include <config/have_getpeerucred.h>
#if defined(HAVE_GETPEEREID)
# include <sys/types.h>
# include <sys/socket.h>
# include <unistd.h>
# include <string.h>
# include <errno.h>
#elif defined(HAVE_SO_PEERCRED)
# include <sys/types.h>
# include <sys/socket.h>
# include <string.h>
# include <errno.h>
#elif defined(HAVE_GETPEERUCRED)
# include <ucred.h>
# include <stdlib.h>
#endif

static __inline__ int
Getpeereid(int sock, uid_t *uid, gid_t *gid)
{
#ifdef HAVE_GETPEEREID
	if (getpeereid(sock, uid, gid) == -1) {
		MySetError("getpeereid: %s", strerror(errno));
		return (-1);
	}
	return (0);
#elif defined(HAVE_SO_PEERCRED)
	struct ucred creds;
	socklen_t socklen;

	socklen = sizeof(creds);
	if (getsockopt(sock, SOL_SOCKET, SO_PEERCRED, &creds, &socklen) != 0) {
		MySetError("SO_PEERCRED: %s", strerror(errno));
		return (-1);
	}
	*uid = (uid_t)creds.uid;
	*gid = (gid_t)creds.gid;
	return (0);
#elif defined(HAVE_GETPEERUCRED)
	ucred_t *creds;
	int rv;

	if ((creds = malloc(ucred_size())) == NULL) {
		MySetError("Out of memory");
		return (-1);
	}
	rv = getpeerucred(sock, &creds);
	if (rv < 0) {
		MySetError("getpeerucred failed");
		return (-1);
	}
	*uid = ucred_geteuid(creds);
	*gid = ucred_getegid(creds);
	ucred_free(creds);
	return (0);
#else
	MySetError("getpeereid not available");
	return (-1);
#endif
}
